import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedHashSet;

/**
 * A specialization of {@link WordCounter} that counts only select words.
 * The list of words that are counted is read from a file by the constructor. 
 */
public class FilteredWordCounter extends WordCounter {
    /**
     * This set implementation allows us both to quickly determine whether we care about a word
     * and maintain the order of the words, which ensures that the output of {@link Count}
     * has the same order as the input.
     */
    private LinkedHashSet<String> wordsOfInterest;               // stored to preserve the input order
        
    /**
     * Read a list of words that should be counted from a file.
     * @param filename the name of the file that contains the word list
     * @throws IOException the provided file could not be read
     */
    public FilteredWordCounter(String filename) throws IOException {
        wordsOfInterest = new LinkedHashSet<String>();
        
        BufferedReader file = new BufferedReader(new FileReader(filename));
        String line;
        
        // Read words from word list file and add them to the set.
        while ((line = file.readLine()) != null)
            wordsOfInterest.add(line);
    }
    
    /** Counts <code>word</code>; does nothing if <code>word</code> is
     * not one of the words that were in the list that was read by the constructor.
     * @param word the word to be counted
     */
    public void count(String word) {
        // Only count words that we actually care about.
        if (wordsOfInterest.contains(word))
            super.count(word);
    }
    
    /** 
     * Returns an iterator that that will yield every word of interest.
     * Words of interest that were never encountered (i.e., that have a count of zero) are included
     * in the iteration.
     */
    public Iterator<String> iterator() {
        return wordsOfInterest.iterator();
    }
}
